import { BG } from "bgutils";
import type { BgConfig } from "bgutils";
import { JSDOM } from "jsdom";
import { Innertube, UniversalCache } from "youtubei.js";
import { Store } from "@willsoto/node-konfig-core";
let getFetchClientLocation = "getFetchClient";
if (Deno.env.get("GET_FETCH_CLIENT_LOCATION")) {
    if (Deno.env.has("DENO_COMPILED")) {
        getFetchClientLocation = Deno.mainModule.replace("src/main.ts", "") +
            Deno.env.get("GET_FETCH_CLIENT_LOCATION");
    } else {
        getFetchClientLocation = Deno.env.get(
            "GET_FETCH_CLIENT_LOCATION",
        ) as string;
    }
}
const { getFetchClient } = await import(getFetchClientLocation);

// Adapted from https://github.com/LuanRT/BgUtils/blob/main/examples/node/index.ts
export const poTokenGenerate = async (
    innertubeClient: Innertube,
    konfigStore: Store<Record<string, unknown>>,
    innertubeClientCache: UniversalCache,
): Promise<Innertube> => {
    const externalTokenGenerator = konfigStore.get(
        "server.external_token_generator",
    ) as string;
    const requestKey = "O43z0dpjhgX20SCx4KAo";

    if (externalTokenGenerator != "" && externalTokenGenerator != undefined) {
        console.log(
            "using external token generator at " + externalTokenGenerator,
        );
        try {
            const response = await fetch(
                `${externalTokenGenerator}/generate`,
            );
            const data = await response.json();
            // innertubeClient.session.po_token = data.potoken;
            // innertubeClient.session.context.client.visitorData =
            //     data.visitorData;
            console.log(data.potoken);
            console.log(data.visitorData);
            return (await Innertube.create({
                po_token: data.potoken,
                visitor_data: data.visitorData,
                fetch: getFetchClient(konfigStore),
                cache: innertubeClientCache,
                generate_session_locally: true,
            }));
        } catch (e) {
            console.error("error fetch token from the external token generator: " + e);
        }
    }
    // setInterval(() => {
    //     fetch(
    //         `${externalTokenGenerator}/generate`,
    //     ).then((response) => response.json()).then((data) => {
    //         innertubeClient.session.po_token = data.potoken;
    //         innertubeClient.session.context.client.visitorData =
    //             data.visitorData;
    //         console.log("po_token: "+data.potoken);
    //         console.log("visitor_data"+data.visitorData);
    //         return (await Innertube.create({
    //             po_token: poTokenResult.poToken,
    //             visitor_data: visitorData,
    //             fetch: getFetchClient(konfigStore),
    //             cache: innertubeClientCache,
    //             generate_session_locally: true,
    //         }));
    //     }).catch(e => {
    //         console.error("error getting tokens from external token generator: " + e)
    //         console.log("tokens will not change")
    //     });
    // }, 5000);

    if (innertubeClient.session.po_token) {
        innertubeClient = await Innertube.create({ retrieve_player: false });
    }

    const visitorData = innertubeClient.session.context.client.visitorData;

    if (!visitorData) {
        throw new Error("Could not get visitor data");
    }

    const dom = new JSDOM();

    Object.assign(globalThis, {
        window: dom.window,
        document: dom.window.document,
    });

    const bgConfig: BgConfig = {
        fetch: getFetchClient(konfigStore),
        globalObj: globalThis,
        identifier: visitorData,
        requestKey,
    };

    const bgChallenge = await BG.Challenge.create(bgConfig);

    if (!bgChallenge) {
        throw new Error("Could not get challenge");
    }

    const interpreterJavascript = bgChallenge.interpreterJavascript
        .privateDoNotAccessOrElseSafeScriptWrappedValue;

    if (interpreterJavascript) {
        new Function(interpreterJavascript)();
    } else throw new Error("Could not load VM");

    const poTokenResult = await BG.PoToken.generate({
        program: bgChallenge.program,
        globalName: bgChallenge.globalName,
        bgConfig,
    });

    await BG.PoToken.generatePlaceholder(visitorData);

    console.log("realpotoken: "+poTokenResult.poToken)
    console.log("realvisitordata: "+visitorData)
    return (await Innertube.create({
        po_token: poTokenResult.poToken,
        visitor_data: visitorData,
        fetch: getFetchClient(konfigStore),
        cache: innertubeClientCache,
        generate_session_locally: true,
    }));
};
