import { Hono } from "hono";
import { routes } from "./routes/index.ts";
import { Innertube, UniversalCache } from "youtubei.js";
import { poTokenGenerate } from "./lib/jobs/potoken.ts";
import { konfigLoader } from "./lib/helpers/konfigLoader.ts";
let getFetchClientLocation = "getFetchClient";
if (Deno.env.get("GET_FETCH_CLIENT_LOCATION")) {
    if (Deno.env.has("DENO_COMPILED")) {
        getFetchClientLocation = Deno.mainModule.replace("src/main.ts", "") +
            Deno.env.get("GET_FETCH_CLIENT_LOCATION");
    } else {
        getFetchClientLocation = Deno.env.get(
            "GET_FETCH_CLIENT_LOCATION",
        ) as string;
    }
}
const { getFetchClient } = await import(getFetchClientLocation);

const app = new Hono();
const konfigStore = await konfigLoader();

let innertubeClient: Innertube;
let innertubeClientFetchPlayer = true;
const innertubeClientOauthEnabled = konfigStore.get(
    "youtube_session.oauth_enabled",
) as boolean;
const innertubeClientJobPoTokenEnabled = konfigStore.get(
    "jobs.youtube_session.po_token_enabled",
) as boolean;
const innertubeClientCookies = konfigStore.get(
    "jobs.youtube_session.cookies",
) as string;
let innertubeClientCache = new UniversalCache(
    true,
    konfigStore.get("cache.directory") as string + "/youtubei.js/",
) as UniversalCache;

Deno.env.set("TMPDIR", konfigStore.get("cache.directory") as string);

if (!innertubeClientOauthEnabled) {
    if (innertubeClientJobPoTokenEnabled) {
        console.log("[INFO] job po_token is active.");
        // Don't fetch fetch player yet for po_token
        innertubeClientFetchPlayer = false;
    } else if (!innertubeClientJobPoTokenEnabled) {
        console.log("[INFO] job po_token is NOT active.");
    }
} else if (innertubeClientOauthEnabled) {
    // Can't use cache if using OAuth#cacheCredentials
    innertubeClientCache = new UniversalCache(false);
}

innertubeClient = await Innertube.create({
    cache: innertubeClientCache,
    retrieve_player: innertubeClientFetchPlayer,
    fetch: getFetchClient(konfigStore),
    cookie: innertubeClientCookies || undefined,
});

if (!innertubeClientOauthEnabled) {
    if (innertubeClientJobPoTokenEnabled) {
        innertubeClient = await poTokenGenerate(
            innertubeClient,
            konfigStore,
            innertubeClientCache as UniversalCache,
        );
    }
    setInterval(
        async () => {
            const currentDateTime = new Date();
            console.log("regenrate token called: " + currentDateTime)
            if (innertubeClientJobPoTokenEnabled) {
                innertubeClient = await poTokenGenerate(
                    innertubeClient,
                    konfigStore,
                    innertubeClientCache,
                );
                const currentDateTimexd = new Date();
            console.log("regenerate token finished: " + currentDateTimexd)
            console.log("po_token: " + innertubeClient.session.po_token)
            } else {
                innertubeClient = await Innertube.create({
                    cache: innertubeClientCache,
                    retrieve_player: innertubeClientFetchPlayer,
                });
            }
        },
        konfigStore.get("jobs.youtube_session.frequency_seconds") as number * 1000 || 50000);
} else if (innertubeClientOauthEnabled) {
    // Fired when waiting for the user to authorize the sign in attempt.
    innertubeClient.session.on("auth-pending", (data) => {
        console.log(
            `[INFO] [OAUTH] Go to ${data.verification_url} in your browser and enter code ${data.user_code} to authenticate.`,
        );
    });
    // Fired when authentication is successful.
    innertubeClient.session.on("auth", () => {
        console.log("[INFO] [OAUTH] Sign in successful!");
    });
    // Fired when the access token expires.
    innertubeClient.session.on("update-credentials", async () => {
        console.log("[INFO] [OAUTH] Credentials updated.");
        await innertubeClient.session.oauth.cacheCredentials();
    });

    // Attempt to sign in and then cache the credentials
    await innertubeClient.session.signIn();
    await innertubeClient.session.oauth.cacheCredentials();
}

app.use("*", async (c, next) => {
    // @ts-ignore Do not understand how to fix this error.
    c.set("innertubeClient", innertubeClient);
    // @ts-ignore Do not understand how to fix this error.
    c.set("konfigStore", konfigStore);
    await next();
});

routes(app, konfigStore);
const https = Deno.env.get("HTTPS");
const port = konfigStore.get("server.port") as number;
const host = konfigStore.get("server.host") as string;


if (https == "TRUE" || https == "true") {
    const cert = Deno.readTextFileSync("/data/cert.pem");
    const key = Deno.readTextFileSync("/data/key.key");
    Deno.serve({
        port: port,
        hostname: host,
        cert: cert,
        key: key,
    }, app.fetch);
} else {
    Deno.serve({
        port: port,
        hostname: host,
    }, app.fetch);
}
